//
//  taskit GmbH Berlin, www.taskit.de
//
//  ViewController.m
//  iOS_taskit_BLE232_Demo001
//
//  Created by Daniel Carl on 17.02.15.
//  Copyright (c) 2015 taskit. All rights reserved.
//

#import "ViewController.h"
#import <CoreBluetooth/CoreBluetooth.h>

#define BLE232_SERVICE_UUID             @"912FFFF0-3D4B-11E3-A760-0002A5D5C51B"
#define BLE232_CHARACTERISTIC_DATA_UUID @"912ffff1-3d4b-11e3-a760-0002a5d5c51b"

@interface ViewController () <CBCentralManagerDelegate, CBPeripheralDelegate>

@property CBCentralManager * centralManager;
@property CBPeripheral * lastDetectedPeripheral;
@property CBCharacteristic * BLE232DataCharactersitic;

@property (weak, nonatomic) IBOutlet UIButton *connectButton;
@property (weak, nonatomic) IBOutlet UIButton *sendButton;
@property (weak, nonatomic) IBOutlet UILabel *centralManagerStateLabel;
@property (weak, nonatomic) IBOutlet UITextView *receivedTextView;
@property (weak, nonatomic) IBOutlet UITextField *sendTextEdit;
@property (weak, nonatomic) IBOutlet UILabel *detectedPeripheralLabel;
@property (weak, nonatomic) IBOutlet UILabel *PeripheralStateLabel;

@end

@implementation ViewController

- (id)init {
    self = [super init];
    if (self) {
        _centralManager = nil;
        _lastDetectedPeripheral = nil;
        _BLE232DataCharactersitic = nil;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark -

#pragma mark View Events

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];

    _centralManagerStateLabel.text = @"Starting ...";
    _receivedTextView.text = @"";
    _sendTextEdit.text = @"";
    _detectedPeripheralLabel.text = @"";
    _PeripheralStateLabel.text = @"";
}

- (void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    // 1. Start Bluetooth Central Manager (in the main / GUI event queue)
    _centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil];
}

- (IBAction)sendTextEditDidEndOnExit:(id)sender {
    [sender resignFirstResponder];
}

#pragma mark -

#pragma mark Bluetooth Central Manager delegates <CBCentralManagerDelegate>

#pragma mark Required

- (void)centralManagerDidUpdateState:(CBCentralManager *)central {
    // 2. Perform actions corresponding to Bluetooth state (info: parameter "central" == property "_centralManager")
    switch (central.state) {
        case CBCentralManagerStatePoweredOn:
            // 3. Start scanning for peripherals. Only show peripherals with BLE232 service.
            /* Note:
               The scan rate will automatically reduced over time by iOS.
               It will be reduced after 22 seconds and even further reduced after 90 seconds.
               Stop the scan and restart it again to have fast peripheral detection.
             */
            _centralManagerStateLabel.text = @"Bluetooth active";
            _receivedTextView.text = @"";
            _detectedPeripheralLabel.text = @"";
            _PeripheralStateLabel.text = @"Scanning ...";
            [_centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:BLE232_SERVICE_UUID]] options:nil];
            break;
        default:
            _centralManagerStateLabel.text = [[NSString alloc] initWithFormat:@"Bluetooth not active (%i)", central.state];
            _receivedTextView.text = @"";
            _detectedPeripheralLabel.text = @"";
            _PeripheralStateLabel.text = @"";
            break;
    }
}

#pragma mark Optional

- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)peripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI {
    // show discovered peripheral
    _detectedPeripheralLabel.text = [NSString stringWithFormat:@"%@ (%i)", peripheral.name, RSSI.intValue];
    // optionally stop scan
    [_centralManager stopScan];
    _centralManagerStateLabel.text = @"Inactive";
    // 4. prepare for peripheral interaction by assigning delegate
    _lastDetectedPeripheral = peripheral;
    _lastDetectedPeripheral.delegate = self;
    // prepare GUI for interaction
    switch (_lastDetectedPeripheral.state) {
        case CBPeripheralStateConnected:
            _PeripheralStateLabel.text = @"Connected";
            break;
        case CBPeripheralStateConnecting:
            _PeripheralStateLabel.text = @"Connecting ...";
            break;
        case CBPeripheralStateDisconnected:
            _PeripheralStateLabel.text = @"Not Connected";
            break;
    }
}

- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral {
    if (peripheral != _lastDetectedPeripheral) return;
    
    _PeripheralStateLabel.text = @"Connected";
    _receivedTextView.text = [NSString stringWithFormat:@"%@ connected.", peripheral.name];
    // 6. discover services (only BLE232 service)
    [_lastDetectedPeripheral discoverServices:@[[CBUUID UUIDWithString:BLE232_SERVICE_UUID]]];
}

- (void)centralManager:(CBCentralManager *)central didFailToConnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    if (peripheral != _lastDetectedPeripheral) return;
    
    _lastDetectedPeripheral.delegate = nil;
    _lastDetectedPeripheral = nil;
    
    _centralManagerStateLabel.text = @"Bluetooth active";
    _receivedTextView.text = [NSString stringWithFormat:@"Connect failed with error: %@", error.description];
    _detectedPeripheralLabel.text = @"-";
    _PeripheralStateLabel.text = @"Scanning ...";
    
    [_centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:BLE232_SERVICE_UUID]] options:nil];
}

- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(NSError *)error {
    if (peripheral != _lastDetectedPeripheral) return;
    
    _lastDetectedPeripheral.delegate = nil;
    _lastDetectedPeripheral = nil;
    _BLE232DataCharactersitic = nil;
    
    _centralManagerStateLabel.text = @"Bluetooth active";
    _receivedTextView.text = [NSString stringWithFormat:@"%@ disconnected.", peripheral.name];
    _detectedPeripheralLabel.text = @"-";
    _PeripheralStateLabel.text = @"Scanning ...";

    [_centralManager scanForPeripheralsWithServices:@[[CBUUID UUIDWithString:BLE232_SERVICE_UUID]] options:nil];
}

#pragma mark -

#pragma mark Bluetooth Peripheral delegates <CBPeripheralDelegate>

#pragma mark Optional

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverServices:(NSError *)error {
    if (peripheral != _lastDetectedPeripheral) return;
    if (error) {
        _receivedTextView.text = [NSString stringWithFormat:@"Discover services failed: %@", error.description];
        return;
    }
    // 7. discover charactersitics (only DATA characteristic of BLE232 service)
    const unsigned serviceCount = _lastDetectedPeripheral.services.count;
    for (unsigned i = 0; i < serviceCount; i++) {
        CBService * service = [_lastDetectedPeripheral.services objectAtIndex:i];
        if ([service.UUID isEqual:[CBUUID UUIDWithString:BLE232_SERVICE_UUID]]) {
            [_lastDetectedPeripheral discoverCharacteristics:@[[CBUUID UUIDWithString:BLE232_CHARACTERISTIC_DATA_UUID]] forService:service];
            break;
        }
    }
}

- (void)peripheral:(CBPeripheral *)peripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {
    if (peripheral != _lastDetectedPeripheral) return;
    if (error) {
        _receivedTextView.text = [NSString stringWithFormat:@"Discover characteristics failed: %@", error.description];
        return;
    }
    // 8. find and store charactersitic (to send data)
    const unsigned Count = service.characteristics.count;
    for (unsigned i = 0; i < Count; i++) {
        CBCharacteristic * characteristic = [service.characteristics objectAtIndex:i];
        if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:BLE232_CHARACTERISTIC_DATA_UUID]]) {
            _BLE232DataCharactersitic = characteristic;
            break;
        }
    }
    // 9. enable notify (to receive data)
    if (_BLE232DataCharactersitic) [_lastDetectedPeripheral setNotifyValue:TRUE forCharacteristic:_BLE232DataCharactersitic];
}

- (void)peripheral:(CBPeripheral *)peripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    if (peripheral != _lastDetectedPeripheral) return;
    if (error) {
        _receivedTextView.text = [NSString stringWithFormat:@"Update value failed: %@", error.description];
        return;
    }
    // receive data
    if (characteristic.value.length == 0) return;
    NSString * newText = [NSString stringWithFormat:@"%@%@", _receivedTextView.text, [[NSString alloc] initWithData:characteristic.value encoding:NSUTF8StringEncoding]];
    _receivedTextView.text = newText;
}

#pragma mark -

#pragma mark User Events
- (IBAction)connectButtonPressed:(id)sender {
    if (_lastDetectedPeripheral == nil) return;
    
    switch (_lastDetectedPeripheral.state) {
        case CBPeripheralStateConnected:
            _PeripheralStateLabel.text = @"Disconnecting ...";
            [_centralManager cancelPeripheralConnection:_lastDetectedPeripheral];
            break;
        case CBPeripheralStateConnecting:
            _PeripheralStateLabel.text = @"Canceling ...";
            [_centralManager cancelPeripheralConnection:_lastDetectedPeripheral];
            break;
        case CBPeripheralStateDisconnected:
            // 5. connect to peripheral
            _PeripheralStateLabel.text = @"Connecting ...";
            [_centralManager connectPeripheral:_lastDetectedPeripheral options:nil];
            break;
    }
}
- (IBAction)sendButtonPressed:(id)sender {
    if (_lastDetectedPeripheral == nil) return;
    
    if (_BLE232DataCharactersitic == nil) return;
    // 10. send data (max 20 bytes at once)
    unsigned Length = _sendTextEdit.text.length;
    if (Length == 0) return;
    
    unsigned Written = 0;
    NSRange Range;
    NSData * Value = [_sendTextEdit.text dataUsingEncoding:NSUTF8StringEncoding];
    
    do {
        Range.location = Written;
        Range.length = (Written + 20) <= Length ? 20 : Length - Written;
        [_lastDetectedPeripheral writeValue:[Value subdataWithRange:Range] forCharacteristic:_BLE232DataCharactersitic type:CBCharacteristicWriteWithResponse];
        Written += Range.length;
    } while (Written < [Value length]);
}

#pragma mark -


@end
